#ifndef __OPTS_HH__
#define __OPTS_HH__

#include "opt.hh"
#include "pudag.hh"
#include "zset.hh"
#include "sched.hh"
#include "grid.hh"
#include "matcher.hh"
#include "snode.hh"
#include "flags.hh"

// #include "nodeattr.hh"
// #include "tp.hh"
// #include "snode.hh"
// #include "tinst.hh"
// #include "archi.hh"
// #include "pattern.hh"

class Optimist : public Optimizer {

public:

  Optimist(Architecture *);
  ~Optimist();
  Schedule *schedule(DAG &, Flags &);
	
private:

  static const int GRID_DIM = 2; 
  static const int T_AXIS = 0;
  static const int L_AXIS = 1;

  typedef grid< SNode *, GRID_DIM > solution_space_t;
  typedef std::vector< unsigned int > coord_t;
#ifdef __GNUC__
  typedef __gnu_cxx::hash_set< SNode *, __gnu_cxx::hash< SNode * >, std::equal_to< SNode * > > grid_elem_t;
#else
  typedef std::hash_set< SNode *, std::hash< SNode * >, std::equal_to< SNode * > > grid_elem_t;
#endif //__GNUC__
  //  solution_space_t solution_space;

//   typedef std::map< TargetInstruction *, std::pair< pattern_t *, std::vector< iso_map_s > > > match_map_t;
//   typedef match_map_t::iterator match_map_iter;


  //  typedef std::map< TargetInstruction *, std::vector< vertex_set_t > > match_map_t;
	
  NodeAttr *get_init_node_attr(DAG &);
  int reduce_init_snode(SNode *, DAG &);
  NodeAttr *select(const vertex_set_t & chi, const NodeAttr &, DAG &);
 
  void update_space(const SNode *, TargetInstruction *,
		    const vertex_set_t &, const coord_t &, 
		    solution_space_t & _solution_space,
		    //vertex_id & _root, 
		    iso_map_s & _iso_maps, //node_map_t & _arg_map,
		    pattern_t & _pattern, DAG _dag);

  coord_t max_coord;
  ZSet *zset;

  //Statistics
  int cut, improved, merged, n_snodes;

  Architecture *architecture;
  Matcher *matcher;
  coord_t last_in;

  typedef bool (*condition)(std::vector< lcc::Node >);
  condition *conditions;

};

// //Equality and hashing for the grid
// #ifdef __GNUC__
// namespace __gnu_cxx 
// #else
// namespace std
// #endif //__GNUC__
// {
//   template<>
//   struct hash< SNode * >
//   {
//     size_t operator()(const SNode * s) const
//     {
//       return s->get_time_profile()->get_hash(); //This is necessary for memory management. Why?
//     }
//   };
// }; //std or __gnu_cxx
// namespace std {
//   template<> 
//   struct equal_to< SNode * >
//   {
//     bool operator()(const SNode * s1, const SNode * s2) const
//     {
//       return *s1 == *s2;
//     }
//   };
// }; //std

#endif //__OPTS_HH__


/*
  bool check_match(pvertex_id & b_u, pattern_t & _pattern, 
  TargetInstruction & _target_instruction, 
  vertex_id & _u_id, DAG & _dag, 
  std::vector< vertex_set_t > & _chi,
  SNode & _current_snode);
*/
//  std::pair<match_map_t::iterator, match_map_t::iterator>

//   std::pair< match_map_iter, match_map_iter >
//   matches(vertex_id & , SNode &, DAG &);

//   match_map_t match_map;

	
/*
  class ZSet {
		
  public:
  ZSet();
  ~ZSet();
		
  NodeAttr *insert(vertex_set_t & _z, indeg_t & _indeg,
  vertex_set_t & _alive);
  NodeAttr *get_node_attr(vertex_set_t & _z);
	
  private:
  typedef std::map< vertex_set_t, NodeAttr * > z_map_t;
  z_map_t z_map;
	
  };
*/


//  std::set< pvertex_id > checked_set;

//pattern checking
//   bool check(pvertex_id _u, pattern_t & _pattern,
// 	     vertex_id _v, DAG & _dag, SNode & _node,
// 	     TargetInstruction & _target_instruction,
// 	     std::vector< iso_map_s > & _iso_maps);
//   bool eq_ops_indeg(pvertex_id _u, pattern_t & _pattern, 
// 	      vertex_id _v, DAG & _dag, NodeAttr & _node_attr);
//   bool eq_out_edges(pvertex_id _u, pattern_t & _pattern,
// 		    vertex_id _v, DAG & _dag, bool *_is_root);
//   bool eq_arg_nodes(pvertex_id & _u, pattern_t & _pattern,
// 		    vertex_id & _v, DAG & _dag,
// 		    TargetInstruction & _target_instruction,
// 		    SNode & _snode,
// 		    node_map_t & _arg_map);

/*
  bool check_parents(pvertex_id _b_child,
  std::vector< pvertex_id > & _b_parents, 
  pattern_t & _pattern, 
  TargetInstruction & _target_instruction, 
  std::vector< vertex_id > & _u_parents, vertex_set_t & _chi, DAG & _dag,
  SNode & _current_snode);
  bool check_other_child(void);
  bool check_children(void);
  bool check_other_parents(void);

  bool check_this_node(pvertex_id & _u, 
  pattern_t & _pattern,
  TargetInstruction & _target_instruction,
  vertex_id & _v,
  DAG & _dag,
  SNode & _current_snode);
*/
