#ifndef __SP_HH__
#define __SP_HH__

/***
 * Space profile  std::map<alive U nodes under compuation, set of int>
 * An alive node can resides in several residence classes simultaneously
 */

#include <iostream>
#include <stack>
#include <map>
#include "tp.hh"
#include "nodeattr.hh"
#include "pudag.hh"
#include "archi.hh"

// An alive node can reside in several residence classes Each entry in
// the set is a pair(residence_class, availability time). Schedules
// refer with their respective reference times. If the reference time
// i larger than the available time (available_at), thus the node is
// not yet available in the asked residence class.
class SpaceProfile : public std::map< vertex_id, std::set< int > >
{

public:
  SpaceProfile(Architecture &);
  SpaceProfile(const SpaceProfile &);
  SpaceProfile & operator=(const SpaceProfile &);
  ~SpaceProfile();
  std::pair< bool, std::pair< char *, lcc::Node > > insert(const vertex_set_t & _alive,
							   const vertex_id & _root, DAG & _dag,
							   int _available_at,
							   int _residence_class, Architecture *_architecture,
							   const std::pair< vertex_id, int > = std::make_pair((vertex_id)-1, 0));

  //If failed, return the earliest instance (must be initialized to -1)
  std::pair< bool, std::pair< char *, lcc::Node > >
  get_instance(vertex_id &, int _residence_class, 
	       int _reference_time, Architecture *, int *_earliest_instance); 
  vertex_set_t get_alive(void) { return alive; }
  friend std::ostream & operator<<(std::ostream &, const SpaceProfile &);

private:
  vertex_set_t alive;
  //Instance map saves for each alive node its residence class, and the instance of the register
  typedef struct { union { int i; char *s; }; lcc::Node p; } instance_t;
  typedef std::pair< int, int > where_when_t;  //residence class, time
  std::map< vertex_id, std::vector< std::pair< where_when_t, instance_t > > > instances_map;
  std::set< vertex_set_t > nodes;
  //  std::map< int, std::set< vertex_set_t > > pressure;  //# of chi (set of nodes) for each rc
  std::vector< std::stack< int > > resources; //resource[residence_class] = satack of reg_index

};

std::ostream & operator<<(std::ostream &, const SpaceProfile &);

#endif //__SP_HH__
