#ifndef __PATTERN_HH__
#define __PATTERN_HH__


#include <boost/config.hpp>
#include <iostream>                  // for std::cout
#include <utility>                   // for std::pair
#include <algorithm>                 // for std::for_each
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <list>
#include "pudag.hh"

using namespace boost;

enum VERTEX_T { 
  ARGNODE, 
  OPNODE 
};

//A vertex has two typed, arg or op_node
struct vertex_kind_t {
  typedef vertex_property_tag kind;
};

struct vertex_info_t {
  typedef vertex_property_tag kind;
};

/*
typedef union {
  struct opnode { 
    int indeg;
    std::list< int > ops; //list of operators (<or>)
  };
  struct argnode {
    int residence; //location of the argument
  };
} info_union_t;
*/
typedef struct pnode { 
  int indeg;
  std::list< int > ops; //list of operators (<or>)
  //  int residence; //location of the argument
  char *id;
  int arg; //argument entry for the condition check
} info_union_t;

typedef property< vertex_kind_t, enum VERTEX_T,
		  property< vertex_info_t, info_union_t > > pattern_vertex_property_t;

//defined in pudag.hh
//enum EDGE_T { ... };

struct pattern_edge_t {
  typedef edge_property_tag kind;
};

typedef property< pattern_edge_t, enum EDGE_T > pattern_edge_property_t;

typedef adjacency_list< vecS, vecS, bidirectionalS,
			pattern_vertex_property_t, 
			pattern_edge_property_t > pattern_t;

typedef graph_traits< pattern_t >::vertex_descriptor pvertex_id;
typedef graph_traits< pattern_t >::edge_descriptor pedge_id;
typedef graph_traits< pattern_t >::in_edge_iterator in_pedge_iter;
typedef graph_traits< pattern_t >::out_edge_iterator out_pedge_iter;
typedef adjacency_iterator_generator< pattern_t, pvertex_id, 
				      out_pedge_iter>::type psuccessor_iterator;

typedef graph_traits< pattern_t>::vertex_iterator pvertex_iter;
typedef property_map< pattern_t, vertex_kind_t > vertex_kind_map;
typedef property_map< pattern_t, vertex_info_t > vertex_info_map;
typedef property_map< pattern_t, pattern_edge_t > pedge_type_map;


vertex_kind_map::type vertex_kind_property(pattern_t &);
vertex_info_map::type vertex_info_property(pattern_t &);
pedge_type_map::type edge_type_property(pattern_t &);

std::pair< psuccessor_iterator, psuccessor_iterator >
psuccessors(pvertex_id & _v_id, pattern_t &);


#endif //__PATTERN_HH__
